import commonJS from '@rollup/plugin-commonjs';
import json from '@rollup/plugin-json';
import resolve from '@rollup/plugin-node-resolve';
import typescript from '@rollup/plugin-typescript';
import type { InputOption, InputOptions } from 'rollup';
import filesize from 'rollup-plugin-filesize';
import litcss from 'rollup-plugin-lit-css';
import styles from 'rollup-plugin-styles';
import transformTaggedTemplate from 'rollup-plugin-transform-tagged-template';
import { visualizer } from 'rollup-plugin-visualizer';
import { transformCSSFragment, transformHTMLFragment } from '../transformers';

const parserOptions = {
  sourceType: 'module',
};

// https://rollupjs.org/configuration-options/
export const getInputOptions = (
  globals: Record<string, string>,
  analyze: boolean,
  open: boolean,
  input: InputOption = 'src/index.ts',
): InputOptions => ({
  context: 'this',
  input,
  // modules which shouldn't be included in the bundle
  external: globals ? Object.keys(globals) : undefined,
  onwarn(warning, warn) {
    // The IIFE export doesn't have a namespace since component exports
    // are expected to be top-level objects
    if (warning.code === 'MISSING_NAME_OPTION_FOR_IIFE_EXPORT') {
      return;
    }

    const nodeModulesNamespaceConflict =
      warning.reexporter && warning.reexporter.indexOf('node_modules') !== -1;
    const nodeModulesCircularDependency =
      warning.importer && warning.importer.indexOf('node_modules') !== -1;

    if (nodeModulesNamespaceConflict || nodeModulesCircularDependency) {
      return;
    }

    warn(warning);
  },
  plugins: [
    resolve({
      browser: true,
    }),
    commonJS(),
    typescript({
      outputToFilesystem: false,
      sourceMap: false,
      tsconfig: 'tsconfig.json',
    }),
    json(),
    styles({ mode: 'emit' }),
    litcss({ specifier: '@microsoft/fast-element' }),
    transformTaggedTemplate({
      tagsToProcess: ['css'],
      transformer: transformCSSFragment,
      parserOptions,
    }),
    transformTaggedTemplate({
      tagsToProcess: ['html'],
      transformer: transformHTMLFragment,
      parserOptions,
    }),
    filesize({
      showMinifiedSize: false,
      showBrotliSize: true,
    }),
    analyze &&
      visualizer((outputOptions) => {
        return {
          title: `Rollup Visualizer - ${outputOptions.file}`,
          filename: outputOptions.file.replace('.js', '.stats.html'),
          gzipSize: true,
          brotliSize: true,
          open,
        };
      }),
  ].filter(Boolean),
});
